#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Replaces Epiphany bookmarks in the local DoudouLinux homepage
# They are located in the DIV element of ID "bookmarks"
#
# Usage:
#  $ ./bookmarks2homepage.py source.html destination.html [bookmark.rdf]
#
# Author: Jean-Michel Philippe <jean-michel.philippe@doudoulinux.org>
# ################################
# This file is part of DoudouLinux.
# 
# DoudouLinux is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# DoudouLinux is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with DoudouLinux.  If not, see <http://www.gnu.org/licenses/>.

#########################
# importations
import sys, os, re, xml.dom
from xml.dom import minidom
from xml.dom.minidom import getDOMImplementation

#########################
# constants
Unclassified = "Unclassified"
HelpCat = [ "Help", "Aide", "Aiuto", "Aiudo", "Hielfe" ]
FaviconURL = "http://a.fvicon.com/%s?canAudit=false&width=16&height=16"

#########################
#########################
#########################
#########################

#########################
class TagDict(dict):
	"""
	Dictionary with sorted keys but Unclassified always put in latest position
	"""
	def keys(self):
		Keys = filter(lambda x: x not in HelpCat, dict.keys(self))
		Keys.sort()
		Help = filter(lambda x: x in HelpCat, dict.keys(self))
		if Help:
			Keys.append( Help[0] )
		
		return Keys

#########################
def sortItemList(ItemList):
	"""
	Sorts an item list by title
	"""
	# collect titles
	TitleList = []
	for Item in ItemList:
		TitleList.append( Item.getElementsByTagName("title")[0].firstChild.data )
	
	# reorder titles
	SortedList = list(TitleList)
	SortedList.sort()
	SortedItems = []
	for Title in SortedList:
		Index = TitleList.index(Title)
		SortedItems.append( ItemList[Index] )
	
	return SortedItems

#########################
def showAsText(ItemDict):
	"""
	Show list of links per subject, text format
	"""
	for Subject in ItemDict.keys():
		print "Subject: ", Subject
		for Item in ItemDict[Subject]:
			Title = Item.getElementsByTagName("title")[0].firstChild.data
			Link = Item.getElementsByTagName("link")[0].firstChild.data
			print "  Item:", Title
			print "       ", Link

#########################
def itemsToHtml(ItemDict, MenuText, MenuTitle):
	"""
	Create Html code for the menu of bookmarks
	"""
	# create the root element
	NewDom = getDOMImplementation().createDocument(None, "div", None)
	DocRoot = NewDom.documentElement
	DocRoot.setAttribute("id", "bookmarks")
	
	# create menu title
	Node = NewDom.createElement("h3")
	Node.setAttribute("title", MenuTitle)
	Node.appendChild(
		NewDom.createTextNode(MenuText)
	)
	DocRoot.appendChild(Node)
	
	# create menu root
	RootUL = NewDom.createElement("ul")
	RootUL.setAttribute("class", "menu")
	DocRoot.appendChild(RootUL)
	
	# loop on subjects for 1st level LI's
	for Subject in ItemDict.keys():
		# insert LI with its SPAN title
		RootLI = NewDom.createElement("li")
		Span = NewDom.createElement("span")
		Span.appendChild(
			NewDom.createTextNode(Subject)
		)
		RootLI.appendChild(Span)
		RootUL.appendChild(RootLI)
		
		# insert 2nd level UL
		MenuUL = NewDom.createElement("ul")
		RootLI.appendChild(MenuUL)
		
		# loop on items for 2nd level LI's
		for Item in ItemDict[Subject]:
			Title = Item.getElementsByTagName("title")[0].firstChild.data
			Link = Item.getElementsByTagName("link")[0].firstChild.data
			
			# create the A link
			Entry = NewDom.createElement("a")
			Entry.setAttribute("href", Link)
			
			# insert image link (favicon)
			Icon = NewDom.createElement("img")
			Icon.setAttribute("src", FaviconURL % Link)
			Icon.setAttribute("alt", "favicon")
			Entry.appendChild(Icon)
			
			# insert text link
			Entry.appendChild(
				NewDom.createTextNode(Title)
			)
			
			# create 2nd level LI and insert the entry
			MenuLI = NewDom.createElement("li")
			MenuLI.appendChild(Entry)
			MenuUL.appendChild(MenuLI)
	
	# output Html code
	# NB: there should not be spaces in the SPAN tag
	Text = NewDom.toprettyxml()[23:]		# NB: removes the xml tag
	Text = re.sub('(<h3>)\s+(.+?)\s+(</h3>)', '\\1\\2\\3', Text)
	Text = re.sub('(<li>)\s+(<span>)\s+(.+?)\s+(</span>)', '\\1\\2\\3\\4', Text)
	Text = re.sub('(<li>)\s+(<a href=)', '\\1\\2', Text)
	Text = re.sub('\s+(</a>)', '\\1', Text)
	return Text

#########################
def seoToHtml(ItemList):
	"""
	Create Html code for the SEO list
	"""
	# create the root element
	NewDom = getDOMImplementation().createDocument(None, "select", None)
	Select = NewDom.documentElement
	Select.setAttribute("id", "search_engine")
	
	# loop on items
	for Item in ItemList:
		Title = Item.getElementsByTagName("title")[0].firstChild.data
		Link = Item.getElementsByTagName("link")[0].firstChild.data
		SmartLink = Item.getElementsByTagName("ephy:smartlink")[0].firstChild.data
		
		# create OPTION tag
		Option = NewDom.createElement("option")
		Option.setAttribute("value", SmartLink.strip())
		if Item == ItemList[0]:
			Option.setAttribute("selected", "selected")
		
		# insert OPTION with text content
		Option.appendChild(
			NewDom.createTextNode(Title)
		)
		Select.appendChild(Option)
	
	# output Html code
	# NB: there should not be spaces in the SPAN tag
	Text = NewDom.toxml()[22:]		# NB: removes the xml tag
	return Text

#########################
#########################
#########################
#########################

#########################
# check and get input args
if len(sys.argv) not in (3, 4):
	print """ERROR: wrong number of arguments

Usage:
  $ python bookmarks2homepage.py SOURCEFILE DESTFILE [BOOKMARKS]

  SOURCEFILE	= template input homepage file, should already exist
  DESTFILE		= output homepage file, created/overwritten
  BOOKMARKS		= Epiphany RDF bookmark file (default: home user's file)
"""
	sys.exit()

InputFile = sys.argv[1]
OutputFile = sys.argv[2]

#########################
# check and read input file
if not os.path.isfile(InputFile):
	print "ERROR: cannot find file '%s'" % InputFile
	sys.exit()
FileId = open(InputFile)
Text = FileId.read().decode("utf-8")
FileId.close()

#########################
# read XML and get item tag elements
if len(sys.argv) == 4:
	FileName = sys.argv[3]
else:
	FileName = os.path.join(os.getenv("HOME"), ".gnome2/epiphany/bookmarks.rdf")
if not os.path.isfile(FileName):
	print "ERROR: cannot find file '%s'" % FileName
	sys.exit()
Filedom = minidom.parse(FileName)
ItemTags = Filedom.getElementsByTagName("item")

#########################
# collect subjects and classify items per subject
ItemGroups = TagDict()
for Item in ItemTags:
	SubjectNode = Item.getElementsByTagName("dc:subject")
	if len(SubjectNode):
		Subject = SubjectNode[0].firstChild.data
	else:
		Subject = Unclassified
	
	# create new subject if needed then add item to subject group
	if Subject not in ItemGroups:
		ItemGroups[Subject] = []
	ItemGroups[Subject].append( Item )

#########################
# collect search engines
SeoList = []
for Item in ItemTags:
	SmartLinkNode = Item.getElementsByTagName("ephy:smartlink")
	if len(SmartLinkNode):
		SeoList.append( Item )

#########################
# sort items in lists and remove lists that only have search engines
for Subject in ItemGroups.keys():
	for Item in ItemGroups[Subject]:
		SmartLinkNode = Item.getElementsByTagName("ephy:smartlink")
		if not len(SmartLinkNode):
			# at least one item is not a search link
			#ItemGroups[Subject] = sortItemList( ItemGroups[Subject] )
			break
	
	# remove this list from menu
	if not len(SmartLinkNode):
		continue
	ItemGroups.pop( Subject )

SeoList = sortItemList( SeoList )
#showAsText(ItemGroups)

#########################
# put bookmarks into the Html code of the source file
Start = Text.index('<div id="bookmarks">')
End = Text.index('</div>', Start)
BkmrkTitle = re.findall('<h3 title="([^"]+)">(.+?)</h3>', Text[Start:End])
if len(BkmrkTitle):
	BkmrkTitle = BkmrkTitle[0]
else:
	BkmrkTitle = ["My preferred websites", "Bookmarks"]
Html = itemsToHtml(ItemGroups, BkmrkTitle[1], BkmrkTitle[0])
Text = Text[:Start] + Html + Text[End+7:]

#########################
# put SEO's into the Html code of the source file
Start = Text.index('<select id="search_engine">')
End = Text.index('</select>', Start)
Html = seoToHtml(SeoList)
Text = Text[:Start] + Html + Text[End+9:]

#########################
# write output file
FileId = open(OutputFile, "w")
FileId.write(Text.encode("utf-8"))
FileId.close()
